==> tinyca-0.4.3.tar.gz: TinyCA/lib/CERT.pm <==
# Copyright (c) Stephan Martin <sm@sm-zone.net>
#
# $Id: CERT.pm,v 1.1.1.1 2002/08/24 20:28:02 sm Exp $
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

use strict;

package CERT;

sub new {
   my $that = shift;
   my $class = ref($that) || $that;

   my $self = {};

   bless($self, $class);
}

sub read_certlist {
   my $self    = shift;
   my $main    = shift;
   my $force   = shift;

   my $ca      = $main->{'CA'}->{'actca'};
   my $certdir = $main->{'CA'}->{$ca}->{'dir'}."/certs";
   my $crlfile = $main->{'CA'}->{$ca}->{'dir'}."/crl/crl.pem";

   my($f, $certlist, $crl, $modt, $parsed);

   $certlist = [];
   
   $modt = (stat($certdir))[9];

   if(defined($self->{'lastread'}) &&
      ($self->{'lastread'} >= $modt) && 
      not defined($force)) {
      return(0);
   }

   $crl = $main->{'OpenSSL'}->parsecrl($main, $crlfile, $force);
   
   opendir(DIR, $certdir) || do {
      $main->print_warning("can't open certdir");
      return(0);
   };

   while($f = readdir(DIR)) {
      next if $f =~ /^\./;

      $f =~ s/\.pem//;
      
      $parsed = $self->parse_cert( $main, $f, $force);

      defined($parsed) || $main->print_error("Can't read Certificate");
      
      my $tmp = MIME::Base64::decode($f);
         $tmp .= "%".$parsed->{'STATUS'};

      push(@{$certlist}, $tmp);
   }
   @{$certlist} = sort(@{$certlist});
   closedir(DIR);

   $self->{'certlist'} = $certlist;

   $self->{'lastread'} = time();
   return(1);  # got new list
}

sub revoke_cert {
   my $self = shift;
   my $main = shift;
   my $ca  = $main->{'CA'}->{'actca'};
   my $ind = $main->{'certtab'}->{'list'}->info('selection');

   my($certname, $cert, $status, $certfile, $passwd, $action);

   if(not defined $ind) {
      $main->print_warning("No Certificate Selected!");
      return;
   }

   ($cert, $status) = split(/\%/, $self->{'certlist'}->[$ind]);

   if($status ne 'VALID') {
      $main->print_warning("Can't revoke Certifikate with Status: $status");
      return;
   }

   $certname = MIME::Base64::encode($cert, '');
   $certfile = $main->{'CA'}->{$ca}->{'dir'}."/certs/".$certname.".pem";

   $action = $main->show_cert_revoke_dialog(\$passwd);

   return if $action eq "Cancel";

   if($main->{'OpenSSL'}->revoke(
         'config' => $main->{'CA'}->{$ca}->{'cnf'},
         'infile' => $main->{'CA'}->{$ca}->{'dir'}."/certs/".$certname.".pem",
         'pass'   => $passwd
         )) {
      $main->print_warning("revoke failed");
      return;
   }

   $main->{'OpenSSL'}->newcrl( 
         'config'  => $main->{'CA'}->{$ca}->{'cnf'},
         'pass'   => $passwd,
         'crldays' => 365,
         'outfile' => $main->{'CA'}->{$ca}->{'dir'}."/crl/crl.pem"
         );

   if (not -s $main->{'CA'}->{$ca}->{'dir'}."/crl/crl.pem") { 
      $main->print_error("newcrl() failed");
   }

   $action = $main->show_key_del_question();

   if($action eq "Yes") {
      unlink($main->{'CA'}->{$ca}->{'dir'}."/keys/".$certname.".pem");
      unlink($main->{'CA'}->{$ca}->{'dir'}."/req/".$certname.".pem");
   }

   # force reread of certlist
   $self->read_certlist($main, 1);

   $main->create_mframe();

   return;
}

sub del_cert {
   my $self = shift;
   my $main = shift;
   my $ca   = $main->{'CA'}->{'actca'};
   my $ind  = $main->{'certtab'}->{'list'}->info('selection');
    
   my($certname, $cert, $certfile, $status);

   if(not defined $ind) {
      $main->print_warning("No Certificate selected!");
      return;
   }

   ($cert, $status) = split(/\%/, $self->{'certlist'}->[$ind]);

   $certname = MIME::Base64::encode($cert, '');
   $certfile = $main->{'CA'}->{$ca}->{'dir'}."/certs/".$certname.".pem";

   if($status eq 'VALID') {
      $main->print_warning("Certificate is not revoked");
      return;
   }

   unlink($certfile);

   # force reread of certlist
   $self->read_certlist($main, 1);

   $main->create_mframe();

   return;
}

sub export_cert {
   my $self = shift;
   my $main = shift;
   my $ca   = $main->{'CA'}->{'actca'};
   my $ind  = $main->{'certtab'}->{'list'}->info('selection');
    
   my($certname, $cert, $certfile, $status, $outfile, $action, 
      $format, $x509, $out, $keyfile, $passwd, $p12passwd, 
      $includeca, $parsed, $cafile);

   if(not defined $ind) {
      $main->print_warning("No Certificate selected");
      return;
   }

   ($cert, $status) = split(/\%/, $self->{'certlist'}->[$ind]);

   $certname = MIME::Base64::encode($cert, '');
   $certfile = $main->{'CA'}->{$ca}->{'dir'}."/certs/".$certname.".pem";

   if($status ne 'VALID') {
      $main->print_warning("Certificate seems not to be valid");
      return;
   }

   $action = $main->show_cert_export_dialog( \$outfile, \$format);

   return if $action eq "Cancel";
   return if(not defined($outfile));

   $parsed = $self->parse_cert($main, $certname);
   
   if($format eq "PEM") {
      $out = $parsed->{'PEM'};
   } elsif ($format eq "DER") {
      $out = $parsed->{'DER'};
   } elsif ($format eq "TXT") {
      $out = $parsed->{'TEXT'};
   } elsif ($format eq "P12") {
      $keyfile = $main->{'CA'}->{$ca}->{'dir'}."/keys/".$certname.".pem";
      $cafile  = $main->{'CA'}->{$ca}->{'dir'}."/cacert.pem";
      if(not -s $keyfile) {
         $main->print_warning("Key is necessary for Export as PKCS#12");
         return;
      }

      $includeca = 0;
      $action = $main->show_p12_export_dialog(
            \$p12passwd,
            \$passwd,
            \$includeca
            );

      return if $action eq "Cancel";
      
      unlink($outfile);
      $main->{'OpenSSL'}->genp12(
            main      => $main,
            certfile  => $certfile,
            keyfile   => $keyfile,
            cafile    => $cafile,
            outfile   => $outfile,
            passwd    => $passwd,
            p12passwd => $p12passwd,
            includeca => $includeca
            );

      if(not -s $outfile) {
         $main->print_warning("genp12() failed");
         return;
      }

      $main->print_info("Certificate and Key exported to $outfile");
      return;

   } else {
      $main->print_warning("Invalid Format");
      return;
   }

   open(OUT, ">$outfile") || do {
      $main->print_warning("Can't open Outfile");
      return;
   };

   print OUT $out;
   close OUT;
   
   $main->print_info("Certificate exported to: $outfile");

   return;
}

sub parse_cert {
   my $self  = shift;
   my $main  = shift;
   my $name  = shift;
   my $force = shift;

   my($ca, $certfile, $x509, $parsed);

   $ca = $main->{'CA'}->{'actca'};

   if($name eq 'CA') {
      $certfile = $main->{'CA'}->{$ca}->{'dir'}."/cacert.pem";
   } else {
      $certfile = $main->{'CA'}->{$ca}->{'dir'}."/certs/".$name.".pem";
   }

   $parsed = $main->{'OpenSSL'}->parsecert(
         $main, 
         $certfile,
         $force
         );

   return($parsed);
}

sub _get_date {
   my $string = shift;

   $string =~ s/  / /g;

   my @t1 = split(/ /, $string);
   my @t2 = split(/:/, $t1[2]);

      $t1[0] = _get_index($t1[0]);

   my $ret = Time::Local::timelocal($t2[2],$t2[1],$t2[0],$t1[1],$t1[0],$t1[3]);

   return($ret);
}

sub _get_index {
   my $m = shift;
   my @a = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);

   for(my $i = 0; $a[$i]; $i++) {
      return $i if($a[$i] eq $m);
   }
}

1

==> tinyca-0.4.5.tar.gz: TinyCA/lib/CERT.pm <==
# Copyright (c) Stephan Martin <sm@sm-zone.net>
#
# $Id: CERT.pm,v 1.5 2003/06/19 13:52:03 sm Exp $
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

use strict;

package CERT;

sub new {
   my $that = shift;
   my $class = ref($that) || $that;

   my $self = {};

   bless($self, $class);
}

sub read_certlist {
   my $self    = shift;
   my $main    = shift;
   my $force   = shift;

   my $ca      = $main->{'CA'}->{'actca'};
   my $certdir = $main->{'CA'}->{$ca}->{'dir'}."/certs";
   my $crlfile = $main->{'CA'}->{$ca}->{'dir'}."/crl/crl.pem";

   my($f, $certlist, $crl, $modt, $parsed, $tmp);

   $certlist = [];
   
   $modt = (stat($certdir))[9];

   if(defined($self->{'lastread'}) &&
      ($self->{'lastread'} >= $modt) && 
      not defined($force)) {
      return(0);
   }

   $crl = $main->{'OpenSSL'}->parsecrl($main, $crlfile, $force);
   
   opendir(DIR, $certdir) || do {
      $main->print_warning("can't open certdir");
      return(0);
   };

   while($f = readdir(DIR)) {
      next if $f =~ /^\./;

      $f =~ s/\.pem//;
      
      $parsed = $self->parse_cert( $main, $f, $force);

      defined($parsed) || $main->print_error("Can't read Certificate");
      
      $tmp = MIME::Base64::decode($f);
      next if not defined($tmp);
      next if $tmp eq "";

      $tmp .= "%".$parsed->{'STATUS'};

      push(@{$certlist}, $tmp);
   }
   @{$certlist} = sort(@{$certlist});
   closedir(DIR);

   $self->{'certlist'} = $certlist;

   $self->{'lastread'} = time();
   return(1);  # got new list
}

sub revoke_cert {
   my $self = shift;
   my $main = shift;
   my $ca  = $main->{'CA'}->{'actca'};
   my $ind = $main->{'certtab'}->{'list'}->info('selection');

   my($certname, $cert, $status, $certfile, $passwd, $action);

   if(not defined $ind) {
      $main->print_warning("No Certificate Selected!");
      return;
   }

   ($cert, $status) = split(/\%/, $self->{'certlist'}->[$ind]);

   if($status ne 'VALID') {
      $main->print_warning("Can't revoke Certifikate with Status: $status");
      return;
   }

   $certname = MIME::Base64::encode($cert, '');
   $certfile = $main->{'CA'}->{$ca}->{'dir'}."/certs/".$certname.".pem";

   $action = $main->show_cert_revoke_dialog(\$passwd);

   return if $action eq "Cancel";

   if($main->{'OpenSSL'}->revoke(
         'config' => $main->{'CA'}->{$ca}->{'cnf'},
         'infile' => $main->{'CA'}->{$ca}->{'dir'}."/certs/".$certname.".pem",
         'pass'   => $passwd
         )) {
      $main->print_warning("revoke failed");
      return;
   }

   $main->{'OpenSSL'}->newcrl(
         $main,
         'config'  => $main->{'CA'}->{$ca}->{'cnf'},
         'pass'    => $passwd,
         'crldays' => 365,
         'outfile' => $main->{'CA'}->{$ca}->{'dir'}."/crl/crl.pem"
         );

   if (not -s $main->{'CA'}->{$ca}->{'dir'}."/crl/crl.pem") { 
      $main->print_error("newcrl() failed");
   }

   $action = $main->show_key_del_question();

   if($action eq "Yes") {
      unlink($main->{'CA'}->{$ca}->{'dir'}."/keys/".$certname.".pem");
      unlink($main->{'CA'}->{$ca}->{'dir'}."/req/".$certname.".pem");
   }

   # force reread of certlist
   $self->read_certlist($main, 1);

   $main->create_mframe();

   return;
}

sub del_cert {
   my $self = shift;
   my $main = shift;
   my $ca   = $main->{'CA'}->{'actca'};
   my $ind  = $main->{'certtab'}->{'list'}->info('selection');
    
   my($certname, $cert, $certfile, $status);

   if(not defined $ind) {
      $main->print_warning("No Certificate selected!");
      return;
   }

   ($cert, $status) = split(/\%/, $self->{'certlist'}->[$ind]);

   $certname = MIME::Base64::encode($cert, '');
   $certfile = $main->{'CA'}->{$ca}->{'dir'}."/certs/".$certname.".pem";

   if($status eq 'VALID') {
      $main->print_warning("Certificate is not revoked");
      return;
   }

   unlink($certfile);

   # force reread of certlist
   $self->read_certlist($main, 1);

   $main->create_mframe();

   return;
}

sub export_cert {
   my $self = shift;
   my $main = shift;
   my $ca   = $main->{'CA'}->{'actca'};
   my $ind  = $main->{'certtab'}->{'list'}->info('selection');
    
   my($certname, $cert, $certfile, $status, $outfile, $action, 
      $format, $x509, $out, $keyfile, $passwd, $p12passwd, 
      $includeca, $parsed, $cafile);

   if(not defined $ind) {
      $main->print_warning("No Certificate selected");
      return;
   }

   ($cert, $status) = split(/\%/, $self->{'certlist'}->[$ind]);

   $certname = MIME::Base64::encode($cert, '');
   $certfile = $main->{'CA'}->{$ca}->{'dir'}."/certs/".$certname.".pem";

   if($status ne 'VALID') {
      $main->print_warning("Certificate seems not to be valid");
      return;
   }

   $action = $main->show_cert_export_dialog( \$outfile, \$format);

   return if $action eq "Cancel";
   return if(not defined($outfile));

   $parsed = $self->parse_cert($main, $certname);
   
   if($format eq "PEM") {
      $out = $parsed->{'PEM'};
   } elsif ($format eq "DER") {
      $out = $parsed->{'DER'};
   } elsif ($format eq "TXT") {
      $out = $parsed->{'TEXT'};
   } elsif ($format eq "P12") {
      $keyfile = $main->{'CA'}->{$ca}->{'dir'}."/keys/".$certname.".pem";
      $cafile  = $main->{'CA'}->{$ca}->{'dir'}."/cacert.pem";
      if(not -s $keyfile) {
         $main->print_warning("Key is necessary for Export as PKCS#12");
         return;
      }

      $includeca = 0;
      $action = $main->show_p12_export_dialog(
            \$p12passwd,
            \$passwd,
            \$includeca
            );

      return if $action eq "Cancel";
      
      unlink($outfile);
      $main->{'OpenSSL'}->genp12(
            main      => $main,
            certfile  => $certfile,
            keyfile   => $keyfile,
            cafile    => $cafile,
            outfile   => $outfile,
            passwd    => $passwd,
            p12passwd => $p12passwd,
            includeca => $includeca
            );

      if(not -s $outfile) {
         $main->print_warning("genp12() failed");
         return;
      }

      $main->print_info("Certificate and Key exported to $outfile");
      return;

   } else {
      $main->print_warning("Invalid Format");
      return;
   }

   open(OUT, ">$outfile") || do {
      $main->print_warning("Can't open Outfile");
      return;
   };

   print OUT $out;
   close OUT;
   
   $main->print_info("Certificate exported to: $outfile");

   return;
}

sub parse_cert {
   my $self  = shift;
   my $main  = shift;
   my $name  = shift;
   my $force = shift;

   my($ca, $certfile, $x509, $parsed);

   $ca = $main->{'CA'}->{'actca'};

   if($name eq 'CA') {
      $certfile = $main->{'CA'}->{$ca}->{'dir'}."/cacert.pem";
   } else {
      $certfile = $main->{'CA'}->{$ca}->{'dir'}."/certs/".$name.".pem";
   }

   $parsed = $main->{'OpenSSL'}->parsecert(
         $main, 
         $certfile,
         $force
         );

   return($parsed);
}

sub _get_date {
   my $string = shift;

   $string =~ s/  / /g;

   my @t1 = split(/ /, $string);
   my @t2 = split(/:/, $t1[2]);

      $t1[0] = _get_index($t1[0]);

   my $ret = Time::Local::timelocal($t2[2],$t2[1],$t2[0],$t1[1],$t1[0],$t1[3]);

   return($ret);
}

sub _get_index {
   my $m = shift;
   my @a = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);

   for(my $i = 0; $a[$i]; $i++) {
      return $i if($a[$i] eq $m);
   }
}

1
